import { Factory, ILifetime } from './ILifetime';
import { ActualProvider, DependencyProvider, NameSelector } from './types';
declare type LifetimeProvider<T, E> = new (futureProvider: () => ActualProvider<E>, dependency: DependencyProvider<T, E>, providerName: string, factoryFunction: Factory<T, E>) => ILifetime<T, E>;
declare type ProviderProvider<E> = {
    prototype: E;
    name: string;
    new (): E;
};
declare type DependencyOptions<T, E> = {
    factory?: Factory<T, E>;
    selector?: NameSelector<T, E>;
};
export declare class Container<E> {
    private static provider?;
    static getOrCreate<T>(factory?: () => Container<T>): ActualProvider<T>;
    static getOrCreateAsync<T>(factory?: () => Promise<Container<T>>): Promise<ActualProvider<T>>;
    readonly template: E;
    private readonly providerLookup;
    private readonly dependencyToProvider;
    private provider?;
    constructor(ProviderTemplate: ProviderProvider<E>);
    addSingleton<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    addTransient<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    add<T>(Lifetime: LifetimeProvider<T, E>, Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    tryAdd<T>(Lifetime: LifetimeProvider<T, E>, Dependency: DependencyProvider<T, E>, { factory, selector }?: DependencyOptions<T, E>): boolean;
    get<T>(item: NameSelector<T, E>): ILifetime<T, E> | undefined;
    remove<T>(item: NameSelector<T, E>): boolean;
    preBuildValidate(): void;
    build(): ActualProvider<E>;
    resolveProperty<T>(item?: NameSelector<T, E>, Dependency?: DependencyProvider<T, E>): string;
}
export {};
