"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Container = void 0;
var InternalProvider_1 = require("./InternalProvider");
var ILifetime_1 = require("./ILifetime");
var DependencyError_1 = require("./DependencyError");
var utils_1 = require("./utils");
var types_1 = require("./types");
var Container = /** @class */ (function () {
    function Container(ProviderTemplate) {
        this.providerLookup = {};
        this.template = new ProviderTemplate();
        this.dependencyToProvider = Object.keys(this.template).reduce(function (a, b) {
            a[b.toLowerCase()] = b;
            return a;
        }, {});
    }
    Container.getOrCreate = function (factory) {
        if (this.provider)
            return this.provider;
        if (!factory)
            throw new Error('Factory not provided and no global provider exists');
        return (this.provider = factory().build());
    };
    Container.getOrCreateAsync = function (factory) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (this.provider)
                            return [2 /*return*/, this.provider];
                        if (!factory)
                            throw new Error('Factory not provided and no global provider exists');
                        _a = this;
                        return [4 /*yield*/, factory()];
                    case 1: return [2 /*return*/, (_a.provider = (_b.sent()).build())];
                }
            });
        });
    };
    Container.prototype.addSingleton = function (Dependency, options) {
        if (options === void 0) { options = {}; }
        this.add(ILifetime_1.Singleton, Dependency, options);
    };
    Container.prototype.addTransient = function (Dependency, options) {
        if (options === void 0) { options = {}; }
        this.add(ILifetime_1.Transient, Dependency, options);
    };
    Container.prototype.add = function (Lifetime, Dependency, options) {
        if (options === void 0) { options = {}; }
        if (!this.tryAdd(Lifetime, Dependency, options)) {
            throw new DependencyError_1.DependencyError({
                type: types_1.DependencyErrorType.Duplicate,
                lifetime: this.resolveProperty(options.selector, Dependency),
            });
        }
    };
    Container.prototype.tryAdd = function (Lifetime, Dependency, _a) {
        var _this = this;
        var _b = _a === void 0 ? {} : _a, _c = _b.factory, factory = _c === void 0 ? function (provider) { return new Dependency(provider); } : _c, selector = _b.selector;
        var providerName = this.resolveProperty(selector, Dependency);
        if (!providerName)
            throw new DependencyError_1.DependencyError({ type: types_1.DependencyErrorType.Unknown, lifetime: Dependency.name });
        if (this.providerLookup[providerName])
            return false;
        this.providerLookup[providerName] = new Lifetime(function () { return _this.build(); }, Dependency, providerName, factory);
        return true;
    };
    Container.prototype.get = function (item) {
        var name = this.resolveProperty(item);
        return this.providerLookup[name];
    };
    Container.prototype.remove = function (item) {
        var name = this.resolveProperty(item);
        if (!this.providerLookup[name])
            return false;
        delete this.providerLookup[name];
        return true;
    };
    Container.prototype.preBuildValidate = function () {
        var _this = this;
        var missing = Object.keys(this.template)
            .filter(function (key) { return !_this.providerLookup[key]; })
            .map(function (key) { return new DependencyError_1.DependencyError({ type: types_1.DependencyErrorType.Existence, lifetime: key }); });
        if (missing.length === 1)
            throw missing[0];
        else if (missing.length > 1)
            throw new DependencyError_1.DependencyMultiError(types_1.DependencyMultiErrorType.Build, missing);
    };
    Container.prototype.build = function () {
        if (this.provider)
            return this.provider;
        var provider = new InternalProvider_1.InternalProvider(this);
        this.preBuildValidate();
        Object.keys(this.template).forEach(function (providerName) {
            Object.defineProperty(provider, providerName, { get: function () { return provider.get(providerName); } });
        });
        return (this.provider = provider);
    };
    Container.prototype.resolveProperty = function (item, Dependency) {
        if (item)
            return typeof item === 'string' ? item : item((0, utils_1.properties)(this.template));
        if (Dependency)
            return this.dependencyToProvider[Dependency.name.toLowerCase()];
        return undefined;
    };
    return Container;
}());
exports.Container = Container;
