"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InternalProvider = void 0;
var DependencyError_1 = require("./DependencyError");
var types_1 = require("./types");
var InternalProvider = /** @class */ (function () {
    function InternalProvider(container) {
        this.container = container;
    }
    InternalProvider.prototype.get = function (nameSelector) {
        var _a;
        var name = this.container.resolveProperty(nameSelector);
        try {
            return (_a = this.container.get(name)) === null || _a === void 0 ? void 0 : _a.provide();
        }
        catch (e) {
            var error = e;
            if (error.message === 'Maximum call stack size exceeded' || error.type === types_1.DependencyErrorType.Circular)
                throw new DependencyError_1.DependencyError({ type: types_1.DependencyErrorType.Circular, lifetime: name });
            throw e;
        }
    };
    InternalProvider.prototype.validate = function () {
        var keys = Object.keys(this.container.template);
        var unresolved = [];
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            try {
                this.get(key);
            }
            catch (e) {
                unresolved.push(e);
            }
        }
        if (unresolved.length === 0)
            return;
        throw new DependencyError_1.DependencyMultiError(types_1.DependencyMultiErrorType.Validation, unresolved);
    };
    return InternalProvider;
}());
exports.InternalProvider = InternalProvider;
